from sharepa.search import ShareSearch, basic_search  # noqa
from sharepa.analysis import bucket_to_dataframe, merge_dataframes  # noqa


def source_counts():
    return bucket_to_dataframe(
        'total_source_counts',
        basic_search.execute().aggregations.sourceAgg.buckets
    )



import json
import requests

from elasticsearch_dsl import Search
from elasticsearch_dsl.result import Response


class ShareSearch(Search):
    BASE_URL = 'https://osf.io/api/v1/share/search/'
    HEADERS = {'content-type': 'application/json'}
    PARAMS = dict(raw=True)

    def execute(self):
        return Response(
            self._query(self.to_dict()),
            callbacks=self._doc_type_map
        )

    def count(self):
        d = self.to_dict()
        if d.get('aggs'):
            del d['aggs']
        self = ShareSearch.from_dict(d)
        return self._query(self.to_dict(), params=dict(count=True))['count']

    def scan(self, size=100):
        count = 0
        page = list(self[0:size].execute())
        while(page):
            for hit in page:
                count += 1
                yield hit
            page = list(self[count:count + size].execute())

    def _query(self, data, params=None):
        return requests.post(
            self.BASE_URL,
            headers=self.HEADERS,
            data=json.dumps(self.to_dict()),
            params=params or self.PARAMS
        ).json()

basic_search = ShareSearch()
basic_search.aggs.bucket(
    'sourceAgg',
    'terms',
    field='_type',
    size=0,
    min_doc_count=0
)



import pandas as pd


def bucket_to_dataframe(name, bucket):
    '''A function that turns elasticsearch aggregation buckets into dataframes

        :param name: The name of the bucket (will be a column in the dataframe)
        :type name: str
        :param bucket: a bucket from elasticsearch results
        :type bucket: list[dict]
        :returns: pandas.DataFrame
    '''
    return pd.DataFrame(
        {item['key']: item['doc_count'] for item in bucket},
        index=[name]
    ).T


def merge_dataframes(*dfs):
    '''A helper function for merging two dataframes that have the same indices

        :param dfs: a list of dataframes to be merged (note: they must have the same indices)
        :type dfs: list[pandas.DataFrame]
        :returns: pandas.DataFrame -- a merged dataframe
    '''
    return pd.concat(dfs, axis=1, join_axes=[dfs[0].index])



from setuptools import setup
from pip.download import PipSession
from pip.req import parse_requirements

VERSION = '0.1.1'

install_reqs = map(
    lambda ir: str(ir.req),
    parse_requirements('requirements.txt', session=PipSession())
)

setup(
    name='sharepa',
    packages=['sharepa'],
    version=VERSION,
    description='A library for browsing and analyzing SHARE data',
    author='Fabian von Feilitzsch',
    author_email='fabian@cos.io',
    url='https://github.com/fabianvf/sharepa',
    download_url='https://github.com/fabianvf/sharepa/tarball/{}'.format(VERSION),
    install_requires=install_reqs
)
